/**
 * 在yonyou平台上实现的具体预处理类 , 需要监听cordova.deviceready事件
 *
 * Created by gengyb on 19/5/16.
 */

import Mutants from './Mutants'
import AndroidRestore from './AndroidRestore'
// import NativeConfig from './native-config';

class EsnBridge{
    
    constructor(){
        this.version = '1.0.0';
        this.initialized = false;
        this.isObject = function(o){
            return Object.prototype.toString.call(o) === '[object Object]';
        }
    }
    
    ready(callback) {
        if (window.WebViewJavascriptBridge) {
            this.initWebViewJavascriptBridge(callback);
        } else {
            document.addEventListener('WebViewJavascriptBridgeReady',
            ()=>{
                this.initWebViewJavascriptBridge(callback);
            },
            false);
        }
    };

    initWebViewJavascriptBridge(callback) {
        if (this.initialized) {
            console.log("init has been called.")
        } else {
            window.WebViewJavascriptBridge.init(function(message, responseCallback) {});
        }
        this.initialized = true ;
        typeof callback === 'function' && callback();
    };

    config(parameters, callback) {
        if(!this.isObject(parameters)){
            throw new Error('参数格式不正确');
        }
        if(!parameters.nonceStr || !parameters.timeStamp || !parameters.jssdkSignature || !parameters.agentId){
            throw new Error('必要参数不存在');
        }
        parameters.fail = function(res){
            console.error(res.error_description || 'config接口调用错误');
        };
        this.do('jsAuthority', parameters, callback);
    };

    registerHandler(parameters) {
        if(!this.isObject(parameters)){
            throw new Error('参数格式不正确');
        }
        for(var key in parameters){
            if(Object.prototype.hasOwnProperty.call(parameters, key)){
                window.WebViewJavascriptBridge.registerHandler(key, parameters[key]);
            }
        }
    };
    
    do(action, parameters) {
        if(action === '' || typeof action !== 'string'){
            throw new Error('参数格式不正确');
        }
        if(!this.isObject(parameters)){
            parameters = {};
        }
        try{
            window.WebViewJavascriptBridge.callHandler(action, parameters, (res)=>{
                // console.log('callback ====== ',parameters);
                var resultData = {};
                try{
                    if(!this.isObject(res)) {
                        res = JSON.parse(res);
                    }
                    if(res.error_code === '0'){
                        resultData = res.data;
                        // console.log('response ====== ','success',resultData);
                        typeof parameters.success === 'function' && parameters.success(resultData);
                    } else {
                        resultData.errCode = res.error_code;
                        resultData.errDescription = res.error_description;
                        // console.log('response ====== ','fail',resultData);
                        typeof parameters.fail === 'function' && parameters.fail(resultData);
                    }
                } catch(e){
                    // resultData.errCode = '1002';
                    // resultData.errDescription = 'data parse error';
                    typeof parameters.fail === 'function' && parameters.fail(resultData);
                }
                typeof parameters.complete === 'function' && parameters.complete();
            });
        } catch(e){
            console.error('e=======',e);
            throw new Error('参数格式不正确');
        }
    };
}
window.YYEsnBridge = new EsnBridge();

class MutantsForYonyou extends Mutants{

    constructor(){
        super();
    }


    _ready(){
        YYEsnBridge.ready(this._deviceReadyHandler.bind(this));
        // document.addEventListener('resume', this._deviceResumeHandler.bind(this));
        // document.addEventListener('pause', this._deviceSuspendHandler.bind(this));

    }

    _deviceReadyHandler(){
        // 获取 native config 信息
        // this._retrieveNativeConfig()
        //     .then(() => {
        //         //如果是安卓, 则需要查询restore状态及数据, 查询完成后, 在进行后续处理
        //         if(this.env.os == this.env.constant.os.android){
        //             AndroidRestore.ready(this._readyCallback.bind(this));
        //         }else{
        //             this._readyCallback();
        //         }
        //     })
        //     .catch(err => {
        //         console.error(err);
        //         this._readyCallback(err);
        //     })

        //如果是安卓, 则需要查询restore状态及数据, 查询完成后, 在进行后续处理
        // if(this.env.os == this.env.constant.os.android){
        //     AndroidRestore.ready(this._readyCallback.bind(this));
        // }else{
            this._readyCallback();
        // }
    }

    // _deviceResumeHandler(){
    //     this.emitter.emit(Mutants.EVENT_NAME.RESUME);
    // }

    // _deviceSuspendHandler(){
    //     this.emitter.emit(Mutants.EVENT_NAME.SUSPEND);
    // }

    _readyCallback(error){
        this._isReady = true;
        this.emitter.emit(Mutants.EVENT_NAME.READY, error);
    }
    
    get isRestore(){
        return AndroidRestore.isRestore;
    }

    get restorePluginData(){
        return AndroidRestore.restorePluginData;
    }

    // _retrieveNativeConfig() {
    //     return new Promise((resolve, reject) => {
    //         const success = (resp) => {
    //             try {
    //                 if (typeof(resp) === 'string') {
    //                     resp = JSON.parse(resp);
    //                 }

    //                 if (resp.data && resp.data.platformExtension && resp.data.platformExtension !== '') {
    //                     // 解析 config 对象
    //                     NativeConfig.parseConfig(resp);
    //                     resolve(NativeConfig.instance);
    //                 }
    //                 else {
    //                     reject('没有得到正确的 config 数据.');
    //                 }
    //             }
    //             catch (err) {
    //                 reject(err.toString());
    //             }
    //         };

    //         const error = (error) => {
    //             reject(error);
    //         };

    //         if (window.cordova && window.cordova.exec) {
    //             // 有 cordova , 调用 native 插件读取 config 信息
    //             window.cordova.exec(success, error, 'Config', 'getConfig', []);
    //         }
    //         else {
    //             reject('没有 cordova 对象, 请确认 cordova 初始化是否正确.');
    //         }
    //     })
    // }
}

export default MutantsForYonyou